package cz.jzitnik.game.entities;

import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import cz.jzitnik.game.annotations.ReduceFallDamage;
import cz.jzitnik.game.core.reducefalldamage.Reducer;

@Getter
@Setter
public class Player implements Serializable {
    private int health = 10;
    private int hunger = 10;
    private int fallDistance = 0;
    private Block playerBlock1;
    private Block playerBlock2;
    private boolean burning = false;
    private int burningTimeout;
    private boolean burningState = false;

    public synchronized void heal() {
        if (hunger > 3 && health < 10) {
            health = Math.min(health + 1, 10);
        }
    }

    public synchronized void drainHunger() {
        if (hunger > 0) {
            hunger--;
        }
    }

    public void addFalling() {
        fallDistance++;
    }

    public void fell(List<Block> fallblock) {
        var block = fallblock.stream().filter(b -> b.getClass().isAnnotationPresent(ReduceFallDamage.class)).findFirst();
        int damage = Math.max(fallDistance - 3, 0);
        if (block.isPresent()) {
            var reducerClass = block.get().getClass().getAnnotation(ReduceFallDamage.class).value();
            try {
				Reducer reducer = reducerClass.getDeclaredConstructor().newInstance();
                damage = reducer.reduce(damage);
			} catch (InstantiationException | IllegalAccessException | IllegalArgumentException
					| InvocationTargetException | NoSuchMethodException | SecurityException e) {
				e.printStackTrace();
                System.exit(0);
			}
        }
        dealDamage(damage);
        fallDistance = 0;
    }

    public synchronized void dealDamage(int amount) {
        health = Math.max(0, health - amount);

        if (health == 0) {
            // TODO: Implement dead
        }
    }

    public synchronized void dealDamage() {
        dealDamage(1);
    }
}
