package cz.jzitnik.game.threads.list;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.blocks.Chest;
import cz.jzitnik.game.blocks.Furnace;
import cz.jzitnik.game.ui.*;
import cz.jzitnik.tui.MouseHandler;
import cz.jzitnik.tui.ScreenRenderer;
import org.jline.terminal.MouseEvent;
import org.jline.terminal.Terminal;
import java.io.IOException;
import java.nio.BufferUnderflowException;
import java.util.Optional;
import cz.jzitnik.game.annotations.ThreadRegistry;

@ThreadRegistry
public class InputHandlerThread extends Thread {
    private final Game game;
    private final Terminal terminal;
    private final ScreenRenderer screenRenderer;
    private final MouseHandler mouseHandler;
    private final boolean[] isRunning;

    public InputHandlerThread(Game game, Terminal terminal, ScreenRenderer screenRenderer, boolean[] isRunning) {
        this.game = game;
        this.terminal = terminal;
        this.screenRenderer = screenRenderer;
        this.mouseHandler = new MouseHandler(game, terminal, screenRenderer);
        this.isRunning = isRunning;
    }

    @Override
    public void run() {
        try {
            while (isRunning[0]) {
                int key = terminal.reader().read();

                if (key == 27) { // ESC
                    key = terminal.reader().read();
                    if (key == '[') {
                        key = terminal.reader().read();
                        if (key == 'M') {
                            MouseEvent mouseEvent = terminal.readMouseEvent();
                            switch (game.getWindow()) {
                                case WORLD -> mouseHandler.handle(mouseEvent);
                                case INVENTORY -> InventoryClickHandler.click(mouseEvent, terminal, screenRenderer,
                                        game, Optional.empty(), Optional.empty());
                                case CRAFTING_TABLE ->
                                        game.getGameStates().craftingTable.click(mouseEvent, terminal, screenRenderer);
                                case CHEST -> ((Chest) game.getWorld()[game.getGameStates().clickY][game
                                        .getGameStates().clickX].stream().filter(i -> i.getBlockId().equals("chest"))
                                                .toList().getFirst().getData()).click(game, mouseEvent, terminal,
                                                        screenRenderer);
                                case FURNACE -> ((Furnace) game.getWorld()[game.getGameStates().clickY][game
                                        .getGameStates().clickX].stream().filter(i -> i.getBlockId().equals("furnace"))
                                                .toList().getFirst().getData()).click(game, mouseEvent, terminal,
                                                        screenRenderer);
                            }
                        }
                    }
                }

                switch (key) {
                    case '1', '2', '3', '4', '5', '6', '7', '8', '9' -> game.changeSlot(key - 49, screenRenderer);
                    case 'a' -> {
                        game.movePlayerLeft(screenRenderer, terminal);
                        screenRenderer.render(game);
                    }
                    case 'd' -> {
                        game.movePlayerRight(screenRenderer, terminal);
                        screenRenderer.render(game);
                    }
                    case ' ' -> {
                        game.movePlayerUp(screenRenderer);
                        screenRenderer.render(game);
                    }
                    case 'e' -> {
                        if (game.getWindow() != Window.WORLD) {
                            game.getInventory().setSelectedItemInv(-1);
                            CloseHandler.handle(game.getWindow(), game);
                            game.setWindow(Window.WORLD);
                        } else {
                            game.setWindow(Window.INVENTORY);
                        }
                        screenRenderer.render(game);
                    }
                    case 'q' -> {
                        if (game.getWindow() != Window.WORLD) {
                            game.setWindow(Window.WORLD);
                        } else {
                            game.setWindow(Window.ESC);
                        }
                        screenRenderer.render(game);
                        System.out.println("Exiting game...");
                        isRunning[0] = false;
                        game.getGameStates().dependencies.gameSaver.save(game);
                        System.exit(0);
                    }
                    default -> {
                    }
                }
            }
        } catch (IOException | BufferUnderflowException e) {
            e.printStackTrace();
        }
    }
}
